import { ref } from 'vue'
import { employeeMapper, getTime } from '../../helpers/employee';
import { fakeData } from '../../helpers/fakedata';
import moment from 'moment';

import 'moment-timezone';


export default {
  setup() {
    const search = ref(null);
    return {
      search
    };
  },
  data: () => ({
    employees: [],
    loading: true,
    secondsToReload: 60,
    cutDay: 6,
    cutHour: 4,
    countdown: 0,
    sinceDate: null,
    interval: null,
    timezone: 'America/Denver',
    searchText: '',
    searchActive: false,
    toDate: null,
    lastWeekMode: false,
    errorMessage: '',
  }),

  computed: {
    hasInfo: function() {
      return this.employees.length === 0 && !this.lastWeekMode && !this.loading && !this.hasError;
    },
    hasMessage: function() {
      return this.lastWeekMode || this.hasInfo || this.hasError;
    },
    hasError: function() {
      return !!this.errorMessage;
    },
    now: function() {
      return moment().tz(this.timezone);
    },
    filteredEmployees: function() {
      if (!this.searchActive || !this.searchText) {
        return this.employees;
      }
      return this.employees.filter(e => {
        const toSearch = `${e.displayName}  ${e.unit} ${e.unit}`.toUpperCase();
        const match = toSearch.indexOf(this.searchText.toUpperCase()) !== -1;
        return match;
      });
    }
  },

  async created() {
    console.log('created');
    await this.initConfig();
    this.reload();
  },

  methods: {
    async initConfig() {
      if (window.services) {
        const config = await window.services.getConfig();
        const {
          secondsToReload = this.secondsToReload,
            cutDay = this.cutDay,
            cutHour = this.cutHour,
            timezone = this.timezone
        } = config;
        this.secondsToReload = secondsToReload;
        this.cutDay = cutDay;
        this.cutHour = cutHour;
        this.timezone = timezone;
      }
    },
    async reload() {
      this.loading = true;
      clearInterval(this.interval);
      await this.fetchData();
      this.now = moment().tz(this.timezone);
      this.setReloadInterval();
      this.loading = false;
    },

    onClickReload() {
      this.reload();
    },

    onClickLastWeekMode() {
      this.lastWeekMode = !this.lastWeekMode;
      this.reload();
    },

    onClickSearch() {
      this.searchActive = !this.searchActive;
      if (this.searchActive) {
        this.search.focus();
      }
    },

    setReloadInterval() {
      this.countdown = this.secondsToReload;
      this.interval = setInterval(() => {
        this.countdown -= 1;
        if (this.countdown === 0) {
          this.reload();
        }
      }, 1000)
    },

    async fetchData() {
      let date = getTime(this.timezone, this.cutDay, this.cutHour);
      let toDate = this.toDate || moment().tz(this.timezone);

      if (this.lastWeekMode) {
        toDate = date.clone().subtract(1, 'second');
        date.subtract(1, 'week');
      }

      this.sinceDate = date.format('MM-DD-YYYY HH:mm:ss');
      this.nowDate = toDate.format('MM-DD-YYYY HH:mm:ss')

      const body = {
        search: {
          s1: {
            field: 'StartTime',
            data: Math.floor(date.toDate().getTime() / 1000),
            type: 'gt',
          },
          s2: {
            field: 'StartTime',
            data: Math.floor(toDate.toDate().getTime() / 1000),
            type: 'le',
          },
          s3: {
            field: 'Discarded',
            data: true,
            type: 'ns'
          }
        },
        join: ['EmployeeObject'],
      };
      let response;

      console.log('body :>> ', body);

      if (window.services) {
        try {
          response = await window.services.getTimesheet(body);
          if (response.status < 200 || response.status >= 300) {
            this.errorMessage = this.getErrorMessage(response.status);
          } else {
            this.employees = employeeMapper(response.body);
            this.errorMessage = '';
          }

          console.log('response :>> ', response);
        } catch (error) {
          console.log('error :>> ', error);
        }
      } else {
        this.employees = employeeMapper(fakeData);
      }

    },
    getErrorMessage(status) {
      if (status === 401) {
        return "Unauthorized, please check token."
      }
      return `Cannot connect to deputy services. (${status})`;
    }
  }
}